package com.ruoyi.demo.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.helper.LoginHelper;
import com.ruoyi.demo.entity.bo.BrandGeoLabelBo;
import com.ruoyi.demo.entity.bo.BrandSearch;
import com.ruoyi.demo.entity.vo.BrandEvolveVo;
import com.ruoyi.demo.entity.vo.ListQueryBody;
import com.ruoyi.demo.entity.vo.OpcloseTimeVo;
import com.ruoyi.demo.service.BrandService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import java.util.List;
import java.util.Map;

@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/brand")
public class BrandController extends BaseController {
    @Autowired
    private BrandService brandService;

    /**
     * 获取品牌搜索推荐
     *
     * @param text 搜索关键字
     * @return
     */
    @GetMapping("/search")
    public R search(String text) {
        return R.ok(brandService.searchByLikeName(text));
    }

    /**
     * 获取品牌信息列表
     *
     * @param body 关键字、网点数量、覆盖城市数和地区码查询请求体
     * @return
     */
    @PostMapping("/list")
    public R list(ListQueryBody body) {
        Page<BrandSearch> brandSearchPage = brandService.searchListByMulti(body, String.valueOf(LoginHelper.getUserId()));
        return R.ok(brandSearchPage);
    }

    /**
     * 获取品牌详细信息
     *
     * @param brandId 品牌id
     * @return
     */
    @GetMapping("/info")
    public R info(String brandId) {
        return R.ok(brandService.searchByBrandId(brandId, String.valueOf(LoginHelper.getUserId())));
    }

    /**
     * 获取品牌物业信息
     *
     * @param brandId 品牌id
     * @return
     */
    @GetMapping("/property")
    public R property(String brandId) {
        return R.ok(brandService.searchPropertyByBrandId(brandId));
    }

    /**
     * 获取品牌地区分布词云
     *
     * @param brandId 品牌id
     * @return
     */
    @GetMapping("/distribution")
    public R distribution(String brandId) {
        return R.ok(brandService.searchDisCloudByBrandId(brandId));
    }

    /**
     * 获取相似品牌
     *
     * @param brandId 品牌id
     * @return
     */
    @GetMapping("/similar")
    public R similar(String brandId) {
        return R.ok(brandService.searchSimilarByBrandId(brandId));
    }

    /**
     * 获取品牌网点营业状态分布
     *
     * @param brandId 品牌id
     * @return
     */
    @GetMapping("/status")
    public R statusDistribution(String brandId) {
        return R.ok(brandService.searchStatusByBrandId(brandId));
    }

    /**
     * 获取品牌网点地图信息
     *
     * @param brandId 品牌id
     * @return
     */
    @GetMapping("/area")
    public R area(String brandId) {
        return R.ok(brandService.searchProvinceByBrandId(brandId));
    }

    /**
     * 获取品牌网点数量及其城市数量分布
     *
     * @param brandId 品牌id
     * @return
     */
    @GetMapping("/shopCount")
    public R shopCount(String brandId) {
        return R.ok(brandService.searchDistributionByBrandId(brandId));
    }

    /**
     * 品牌网点数量对比
     *
     * @param pageSize 返回数据大小
     * @return
     */
    @GetMapping("/total")
    public R total(Integer pageSize) {
        return R.ok(brandService.searchTotal(pageSize));
    }

    /**
     * 获取品牌地理位置标签分布
     *
     * @param brandId 品牌id
     * @return
     */

    @GetMapping("/geolabel")
    public R geolabel(String brandId) {
        List<BrandGeoLabelBo> brandGeoLabelBos = brandService.searchGeoLabel(brandId);
        return R.ok(brandGeoLabelBos);
    }

    /**
     * 行业连锁品牌发展情况
     *
     * @param body 地址码、分类码和品牌ids查询请求体
     * @return
     */

    @PostMapping("/evolve")
    public R develop(@RequestBody ListQueryBody body) {
        List<String> addrCode = body.getAddrCode();
        List<String> brandId = body.getBrandId();
        if (brandId == null || brandId.isEmpty()) return R.fail("brandId不允许为空");

        Map<String, List<BrandEvolveVo>> listMap = brandService.searchEvolveByMulti2Map(addrCode, null, brandId);

        return R.ok(listMap);

    }

    /**
     * 品牌各地区网点开店时间分布情况
     *
     * @param body 地址码、分类码和品牌ids查询请求体
     * @return
     */
    @PostMapping("/opentime")
    public R openTime(@RequestBody ListQueryBody body) {
        List<String> brandIds = body.getBrandId();
        List<String> addrCodes = body.getAddrCode();
        List<String> typeCodeBys = body.getTypeCodeBy();
        List<OpcloseTimeVo> vos = brandService.searchOpenTims(brandIds, addrCodes, typeCodeBys);
        return R.ok(vos);
    }

    /**
     * 品牌各地区网点关店时间分布情况
     *
     * @param body 地址码、分类码和品牌ids查询请求体
     * @return
     */
    @PostMapping("/closetime")
    public R closeTime(@RequestBody ListQueryBody body) {
        List<String> brandIds = body.getBrandId();
        List<String> addrCodes = body.getAddrCode();
        List<String> typeCodeBys = body.getTypeCodeBy();
        List<OpcloseTimeVo> vos = brandService.searchCloseTims(brandIds, addrCodes, typeCodeBys);
        return R.ok(vos);
    }

    /**
     * 城市等级分布
     *
     * @param brandId 品牌id
     * @return
     */
    @GetMapping("/level")
    public R level(@NotBlank(message = "品牌id不能为空") String brandId) {
        return R.ok(brandService.getCityTier(brandId));
    }

    /**
     * 行业新品牌
     *
     * @param years 范围/年
     * @return
     */
    @GetMapping("/new")
    public R newBrand(Integer years) {
        return R.ok(brandService.searchNewByYears(years));
    }
}
