package com.ruoyi.demo.utils;

import cn.hutool.core.lang.Console;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.utils.redis.RedisUtils;
import com.ruoyi.demo.constant.RedisContant;
import com.ruoyi.demo.entity.WdInfo;
import com.ruoyi.demo.mapper.WdInfoDao;
import org.apache.ibatis.cursor.Cursor;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.redisson.api.GeoUnit;
import org.redisson.api.RFuture;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;

@Component
public class WdRedisStoreage {

    @Autowired
    public WdInfoDao wdInfoDao;

    @Autowired
    private SqlSessionFactory sqlSessionFactory;

    @Autowired
    @Qualifier("executor")
    private ThreadPoolTaskExecutor executorService;


    /***
     * 获取中心网点的周边标签
     * @param wdInfo 中心网点信息
     * @return
     */
    public List<String> getWdTag(WdInfo wdInfo) {
        List<String> tag = RedisUtils.getCacheMapValue(RedisContant.WD_TAG_MAP, RedisContant.WD_TAG + "_" + wdInfo.getWdId());
        //List<String> tag = (List<String>) redisTemplate.boundValueOps(RedisContant.WD_TAG+"_"+wdInfo.getWdId()).get();
        return tag;
    }

    public List<List<String>> getWdTagList(List<String> key) {
        Map<String, List<String>> tags = RedisUtils.getMultiCacheMapValue(RedisContant.WD_TAG_MAP, new HashSet<>(key));
        return new ArrayList<>(tags.values());
    }

    /***
     * 分析网点周边标签，并把分析结果存入redis（默认范围：1km）
     * @param wdInfo 中心网点信息
     */
    public void wdTagAnalyse(WdInfo wdInfo) throws ExecutionException, InterruptedException {
        //1. 找到距离中心网点半径1km的所有网点Id
        RFuture<Map<Object, Double>> radius = RedisUtils.getClient().getGeo(wdInfo.getAddrCode().substring(0, 4)).radiusWithDistanceAsync(wdInfo.getWdId(), 1, GeoUnit.KILOMETERS);
        CompletableFuture<List<String>> completableFuture = CompletableFuture.supplyAsync(() -> {
            try {
                Map<Object, Double> distanceMap = radius.get();
                List<String> aroundWdId = distanceMap.keySet().stream().map(item -> (String) item).collect(Collectors.toList());
                //2. 查询周边网点标签的Set集
                QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
                queryWrapper.select("type_name_by")
                    .in("wd_id", aroundWdId)
                    .groupBy("type_name_by");
                List<WdInfo> wdInfos = wdInfoDao.selectList(queryWrapper);
                //3. 打标签
                List<String> tag = new ArrayList<>();
                wdInfos.forEach(item -> {
                    if (item != null) {
                        String typeNameBy = item.getTypeNameBy();
                        if (StrUtil.isNotEmpty(typeNameBy)) {
                            String[] split = typeNameBy.split(":");
                            if (split.length > 0)
                                tag.add(split[split.length - 1]);
                        }
                    }
                });

                RedisUtils.setCacheMapValue(RedisContant.WD_TAG_MAP, RedisContant.WD_TAG + "_" + wdInfo.getWdId(), tag);
                return tag;
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }, executorService);

        completableFuture.get();

    }

    public void writeWdGeoRedis1() throws Exception {
        //1.将网点坐标存入redis中
        long start = System.currentTimeMillis();
        Console.log("开始将网点坐标存入redis");
        //从Redis中获取时间
        String time = RedisUtils.getCacheObject(RedisContant.WD_TAG_LASTTIME);

        //从数据库中获取数据
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
        if (StrUtil.isNotEmpty(time)) {
            queryWrapper.ge("collect_time", time);
        }
        Long total = wdInfoDao.selectCount(queryWrapper);

        queryWrapper.select("wd_id", "addr_code", "lng", "lat");

        //使用sqlSessionFactory打开一个sqlSession，在没有读取完数据之前不要提交事务或关闭sqlSession
        Console.log("----开启sqlSession");
        SqlSession sqlSession = sqlSessionFactory.openSession();
        Cursor<WdInfo> cursor = null;
        try {
            //获取到指定mapper
            WdInfoDao mapper = sqlSession.getMapper(WdInfoDao.class);
            //调用指定mapper的方法，返回一个cursor
            cursor = mapper.streamQuery(queryWrapper);

            if (cursor != null) {
                for (WdInfo wdInfo : cursor) {
                    //打印进度条
                    Console.print("{}当前进度：{} {}/{}", '\r', StrUtil.repeat('#', (int) (50 * (cursor.getCurrentIndex() + 1) / total)), cursor.getCurrentIndex() + 1, total);

                    RedisUtils.getClient().getGeo(wdInfo.getAddrCode().substring(0, 4)).add(wdInfo.getLng().doubleValue(), wdInfo.getLat().doubleValue(), wdInfo.getWdId());
                    wdTagAnalyse(wdInfo);
                    //存入已处理网点的时间
                    RedisUtils.setCacheObject(RedisContant.WD_TAG_LASTTIME, wdInfo.getCollectTime());
                }
                if (cursor.isConsumed()) {
                    Console.log("----查询sql匹配中的数据已经消费完毕！");
                }
            }
            sqlSession.commit();
            Console.log("----提交事务");
        } catch (Exception e) {
            Console.log(e, "数据消费异常");
            sqlSession.rollback();
        } finally {
            if (cursor != null && cursor.isOpen()) {
                //关闭cursor
                cursor.close();
                Console.log("----关闭cursor");
            }
            if (sqlSession != null) {
                //全部数据读取并且做好其他业务操作之后，提交事务并关闭连接；
                sqlSession.close();
                Console.log("----关闭sqlSession");
            }

        }

        long end = System.currentTimeMillis();
        Console.log("完成存入 耗时：" + (end - start) / (1000 * 60) + "分钟");
    }

    public void writeWdGeoRedis2(String[] wdTypeCodes) throws IOException {
        //对所有网点坐标进行分析打标签
        long start = System.currentTimeMillis();
        Console.log("开始打标签");

        //开始
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("wd_type_code", wdTypeCodes);
        Long total = wdInfoDao.selectCount(queryWrapper);

        queryWrapper.select("wd_id", "addr_code");


        //使用sqlSessionFactory打开一个sqlSession，在没有读取完数据之前不要提交事务或关闭sqlSession
        Console.log("----开启sqlSession");
        SqlSession sqlSession = sqlSessionFactory.openSession();
        Cursor<WdInfo> cursor = null;
        try {
            //获取到指定mapper
            WdInfoDao mapper = sqlSession.getMapper(WdInfoDao.class);
            //调用指定mapper的方法，返回一个cursor
            cursor = mapper.streamQuery(queryWrapper);

            if (cursor != null) {
                for (WdInfo wdInfo : cursor) {
                    //打印进度条
                    Console.print("{}当前进度：{} {}/{}", '\r', StrUtil.repeat('#', (int) (50 * (cursor.getCurrentIndex() + 1) / total)), cursor.getCurrentIndex() + 1, total);

                    wdTagAnalyse(wdInfo);
                }
                if (cursor.isConsumed()) {
                    Console.log("----查询sql匹配中的数据已经消费完毕！");
                }
            }
            sqlSession.commit();
            Console.log("----提交事务");
        } catch (Exception e) {
            Console.log(e, "数据消费异常");
            sqlSession.rollback();
        } finally {
            if (cursor != null && cursor.isOpen()) {
                //关闭cursor
                cursor.close();
                Console.log("----关闭cursor");
            }
            if (sqlSession != null) {
                //全部数据读取并且做好其他业务操作之后，提交事务并关闭连接；
                sqlSession.close();
                Console.log("----关闭sqlSession");
            }

        }

        long end = System.currentTimeMillis();
        Console.log("完成打标签 耗时：" + (end - start) / (1000 * 60) + "分钟");

    }


}
